/*
 * CPU-specific routines originally from U-Boot.
 * See: uboot-xburst/files/arch/mips/cpu/xburst/cpu.c
 * See: u-boot/arch/mips/include/asm/cacheops.h
 *
 * Copyright (C) 2000-2009 Wolfgang Denk, DENX Software Engineering, <wd@denx.de>
 * Copyright (C) 2015, 2016 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "mips.h"
#include "paging.h"
#include "sdram.h"

.text
.globl flush_icache_tag
.globl flush_icache_region
.globl flush_icache_config
.globl flush_dcache_region
.globl handle_error_level
.globl enable_interrupts
.globl init_interrupts
.globl invoke_task
.globl configure_tlb
.globl map_page_set_index
.globl map_page_index_op
.globl map_page_op
.set noreorder



flush_icache_tag:
	mtc0 $zero, CP0_TAGLO
	mtc0 $zero, CP0_TAGHI
	jr $ra
	nop



flush_icache_region:
	cache Index_Store_Tag_I, 0($a0)
	jr $ra
	nop



flush_icache_config:
	move $t3, $zero
	mfc0 $t3, CP0_CONFIG, 7
	nop
	ori $t3, 2
	mtc0 $t3, CP0_CONFIG, 7
	jr $ra
	nop



flush_dcache_region:
	cache Index_Writeback_Inv_D, 0($a0)
	jr $ra
	nop



handle_error_level:
	mfc0 $t3, CP0_STATUS
	li $t4, 0xfffffffb	/* ERL = 0 */
	and $t3, $t3, $t4
	mtc0 $t3, CP0_STATUS
	jr $ra
	nop



enable_interrupts:
	mfc0 $t3, CP0_STATUS
	li $t4, 0x0000fc01	/* IE = enable interrupts */
	or $t3, $t3, $t4
	mtc0 $t3, CP0_STATUS
	jr $ra
	nop



init_interrupts:

	/* Set exception registers. */

	mtc0 $zero, CP0_WATCHLO
	li $t3, 0x00800000	/* IV = 1 (use 0x80000200 for interrupts) */
	mtc0 $t3, CP0_CAUSE
	mfc0 $t4, CP0_STATUS
	li $t3, 0xffbfffff	/* BEV=0 */
	and $t3, $t3, $t4
	mtc0 $t3, CP0_STATUS
	jr $ra
	nop



/* (u8 asid, u32 *base, u32 *stack_pointer) */

invoke_task:
	mtc0 $a0, CP0_ENTRYHI
	nop
	move $t4, $a2		/* use arguments before they are overwritten */
	lw $sp, 0($t4)		/* set the stack pointer */
	move $t3, $a1		/* load parameters from the stored registers */
	lw $a0, 16($t3)
	lw $a1, 20($t3)
	lw $a2, 24($t3)
	lw $a3, 28($t3)
	lw $t9, 100($t3)
	lw $gp, 104($t3)
	mtc0 $t9, CP0_EPC
	mfc0 $t3, CP0_STATUS
	ori $t3, $t3, 0x2	/* EXL = 1 */
	mtc0 $t3, CP0_STATUS
	eret
	nop



configure_tlb:
	mtc0 $zero, CP0_CONTEXT
	mtc0 $zero, CP0_WIRED	/* first random entry is zero */
	mfc0 $v0, CP0_CONFIG	/* return the limit */
	jr $ra
	nop



/* (u32 index) */

map_page_set_index:
	mtc0 $a0, CP0_INDEX
	jr $ra
	nop



/* (u32 lower, u32 upper, u32 start, u32 pagemask) */

map_page_index_op:
	jal map_page_op_setup
	nop

	tlbwi
	jr $ra
	nop



/* (u32 lower, u32 upper, u32 start, u32 pagemask) */

map_page_op:
	jal map_page_op_setup
	nop

	tlbwr
	jr $ra
	nop



/* (u32 lower, u32 upper, u32 start, u32 pagemask) */

map_page_op_setup:
	mtc0 $a3, CP0_PAGEMASK

	/* Set physical address. */

	mtc0 $a0, CP0_ENTRYLO0
	mtc0 $a1, CP0_ENTRYLO1

	/* Set virtual address. */

	mtc0 $a2, CP0_ENTRYHI
	jr $ra
	nop



.set reorder
